@@ -1,6 +1,6 @@
 #Function: ghap.blockstats
 #License: GPLv3 or later
-#Modification date: 22 Jan 2016
+#Modification date: 18 Feb 2017
 #Written by: Yuri Tani Utsunomiya
 #Contact: ytutsunomiya@gmail.com
 #Description: Calculate block summary statistics
@@ -12,13 +12,24 @@ ghap.blockstats <- function(hapstats, ncores = 1){
   
   #Set of internal functions
   my.fun <- function(block){
-    temp<-hapstats$FREQ[hapstats$BLOCK==block]
-    exp.het <- 1-(sum((temp)^2))
-    return(c(exp.het,length(temp)))
+    freq <- hapstats$FREQ[hapstats$BLOCK==block & hapstats$TYPE != "ABSENT"]
+    if(sum(freq) == 1){
+      exp.het <- 1-(sum((freq)^2))
+    }else{
+      exp.het <- NA
+    }
+    return(c(exp.het,length(freq)))
   }
     
   #Calculation of expected heterozygosity
-  temp <- unlist(mclapply(FUN = my.fun, blocks$BLOCK, mc.cores = ncores))
+  ncores <- min(c(detectCores(),ncores))
+  if(Sys.info()["sysname"] == "Windows"){
+    cat("\nParallelization not supported yet under Windows (using a single core).\n")
+    temp <- lapply(FUN = my.fun, X = blocks$BLOCK)
+  }else{
+    temp <- mclapply(FUN = my.fun, X = blocks$BLOCK, mc.cores = ncores)
+  }
+  temp <- unlist(temp)
   blocks$EXP.H <- temp[1:length(temp) %% 2 == 1]
   blocks$N.ALLELES <- temp[1:length(temp) %% 2 == 0]
   
